__author__ = 'fcanas'
PARENT_CLOSING_TAG = 'parent_closing_tag'
PARENT_OPENING_TAG = 'parent_opening_tag'
PATTERNS = 'patterns'
SPEC_ELEMENT = 'spec_element'
ATTRIBUTES = 'attributes'
CHECK_KEY_PATTERN = 'check_key_pattern'
SOURCE_SEARCH_PATTERN = 'source_search_pattern'
REFERENCE_SPEC_FILES = 'reference_spec_files'
DEFINITION_SPEC_FILES = 'definition_spec_files'
WRITE_PATTERNS = 'write_patterns'
READ_PATTERNS = 'read_patterns'
WHITE_LIST = 'white_list'
NAME = 'name'
WHITE_LIST_PATTERNS = 'white_list_patterns'
KEY_PATTERN = 'get_key_pattern'
REF_ID_ATTRIBUTES_LIST = 'ref_id_attributes_list'
IGNORE_KEY_PATTERN = 'ignore_key_pattern'

